from ip_manager.testing import ComponentReferenceTest, Compare, CircuitModelTest
import pytest
import numpy as np


@pytest.mark.comparisons([Compare.GdsToGds, Compare.LayoutToXML, Compare.NetlistToXML, Compare.SMatrix])
class TestDiskResonatorReference(ComponentReferenceTest):
    @pytest.fixture
    def component(self):
        # Create and return a basic disk resonator
        from team_library.all import DiskResonator

        my_disk = DiskResonator(name="my_disk")
        return my_disk

    @pytest.fixture
    def wavelengths(self):
        return np.arange(1.3, 1.8, 0.0001)


# You can also write your own circuit model tests.
# In the example below, we test passivity, reciprocity, peak location and transmission values.
class TestDiskResonatorPeaks(CircuitModelTest):
    @pytest.fixture
    def component(self, disk_n_eff, disk_rt_trans):
        # Create and return a basic disk resonator
        from team_library.all import DiskResonator

        my_disk = DiskResonator(name="my_disk2")
        my_disk.CircuitModel(
            disk_n_eff=disk_n_eff,
            disk_roundtrip_transmission=disk_rt_trans,
        )
        return my_disk

    # The reference file name has to differentiate between different parameter values.
    @pytest.fixture
    def reference_name(self, disk_n_eff, disk_rt_trans):
        return "my_disk_" + str(disk_n_eff) + "_" + str(disk_rt_trans)

    # This test will run 4 times in total, once for every set of parameters
    # (which are auto-converted to fixtures during the test).
    @pytest.mark.parametrize(
        "disk_n_eff, disk_rt_trans, exp_peak_wl, exp_peak_fwhm",
        [
            (3.00, 0.92, 1.500, 0.0035),  # noqa
            (3.01, 0.88, 1.505, 0.0045),  # noqa
            (3.02, 0.84, 1.510, 0.0057),  # noqa
            (3.03, 0.80, 1.515, 0.0067),
        ],
    )  # noqa
    def test_disk_smatrix(self, smatrix, disk_n_eff, disk_rt_trans, exp_peak_wl, exp_peak_fwhm):

        # Create a tester object
        from ip_manager.testing import SMatrixTester

        tester = SMatrixTester(smatrix)

        # Perform general tests
        assert tester.is_reciprocal(), "Component is not reciprocal"
        assert tester.is_passive(), "Component is active"

        # Select a terminal link
        tester.select_ports("in", "out")  # from 'in' to 'out'

        # Perform tests on this link
        assert tester.has_peak_at(centre=exp_peak_wl, fwhm=exp_peak_fwhm), f"No peak at {exp_peak_wl} um"
        assert tester.powerdB_at(exp_peak_wl) < -6.0, "Peak is not deep enough"
